@@ -19,7 +19,9 @@
    Boston, MA 02111-1307, USA.  */
 
 #include "system.h"
+#include "quotearg.h"
 #include "files.h"
+#include "symtab.h"
 #include "gram.h"
 #include "LR0.h"
 #include "lalr.h"
@@ -28,6 +30,7 @@
 #include "getargs.h"
 #include "state.h"
 #include "reader.h"
+#include "closure.h"
 #include "obstack.h"
 #include "print_graph.h"
 #include "vcg.h"
@@ -35,42 +38,74 @@
 static graph_t graph;
 static FILE *fgraph = NULL;
 
-/* This part will construct the label of nodes. */
+
+/*----------------------------.
+| Construct the node labels.  |
+`----------------------------*/
+
 static void
-print_core (int state, struct obstack *node_obstack)
+print_core (struct obstack *oout, state_t *state)
 {
   int i;
-  core *statep = state_table[state].state;
+  item_number_t *sitems = state->items;
+  int snritems   = state->nitems;
 
-  if (!statep->nitems)
-    return;
-
-  for (i = 0; i < statep->nitems; i++)
+  /* Output all the items of a state, not only its kernel.  */
+  if (report_flag & report_itemsets)
     {
-      short *sp;
-      short *sp1;
-      int rule;
+      closure (sitems, snritems);
+      sitems = itemset;
+      snritems = nritemset;
+    }
 
-      sp1 = sp = ritem + statep->items[i];
+  obstack_fgrow1 (oout, "state %2d\n", state->number);
+  for (i = 0; i < snritems; i++)
+    {
+      item_number_t *sp;
+      item_number_t *sp1;
+      rule_number_t rule;
 
-      while (*sp > 0)
+      sp1 = sp = ritem + sitems[i];
+
+      while (*sp >= 0)
 	sp++;
 
-      rule = -(*sp);
+      rule = item_number_as_rule_number (*sp);
 
       if (i)
-	obstack_1grow (node_obstack, '\n');
-      obstack_fgrow1 (node_obstack, "%d: ", rule);
-      obstack_fgrow1 (node_obstack, " %s  ->  ",
-		      tags[rule_table[rule].lhs]);
+	obstack_1grow (oout, '\n');
+      obstack_fgrow1 (oout, " %s -> ",
+		      rules[rule].lhs->tag);
 
-      for (sp = ritem + rule_table[rule].rhs; sp < sp1; sp++)
-	obstack_fgrow1 (node_obstack, "%s ", tags[*sp]);
+      for (sp = rules[rule].rhs; sp < sp1; sp++)
+	obstack_fgrow1 (oout, "%s ", symbols[*sp]->tag);
 
-      obstack_1grow (node_obstack, '.');
+      obstack_1grow (oout, '.');
 
-      for (/* Nothing */; *sp > 0; ++sp)
-	obstack_fgrow1 (node_obstack, " %s", tags[*sp]);
+      for (/* Nothing */; *sp >= 0; ++sp)
+	obstack_fgrow1 (oout, " %s", symbols[*sp]->tag);
+
+      /* Experimental feature: display the lookaheads. */
+      if (report_flag & report_lookaheads)
+	{
+	  /* Find the reduction we are handling.  */
+	  reductions_t *reds = state->reductions;
+	  int redno = state_reduction_find (state, &rules[rule]);
+
+	  /* Print them if there are.  */
+	  if (reds->lookaheads && redno != -1)
+	    {
+	      bitset_iterator biter;
+	      int k;
+	      int not_first = 0;
+	      obstack_sgrow (oout, "[");
+	      BITSET_FOR_EACH (biter, reds->lookaheads[redno], k, 0)
+		obstack_fgrow2 (oout, "%s%s",
+				not_first++ ? ", " : "",
+				symbols[k]->tag);
+	      obstack_sgrow (oout, "]");
+	    }
+	}
     }
 }
 
@@ -81,105 +116,43 @@ print_core (int state, struct obstack *node_obstack)
 `---------------------------------------------------------------*/
 
 static void
-print_actions (int state, const char *node_name)
+print_actions (state_t *state, const char *node_name)
 {
   int i;
 
-  shifts   *shiftp = state_table[state].shifts;
-  reductions *redp = state_table[state].reductions;
-#if 0
-  errs       *errp = err_table[state];
-#endif
+  transitions_t     *transitions = state->transitions;
+  reductions_t *redp = state->reductions;
 
   static char buff[10];
   edge_t edge;
 
-  if (!shiftp->nshifts && !redp)
-    {
-#if 0
-      if (final_state == state)
-	obstack_sgrow (node_obstack, "$default: accept");
-      else
-	obstack_sgrow (node_obstack, "NO ACTIONS");
-#endif
-      return;
-    }
+  if (!transitions->num && !redp)
+    return;
 
-  for (i = 0; i < shiftp->nshifts; i++)
-    if (!SHIFT_IS_DISABLED (shiftp, i))
+  for (i = 0; i < transitions->num; i++)
+    if (!TRANSITION_IS_DISABLED (transitions, i))
       {
-	int state1 = shiftp->shifts[i];
-	int symbol = state_table[state1].accessing_symbol;
+	state_t *state1 = transitions->states[i];
+	symbol_number_t symbol = state1->accessing_symbol;
 
 	new_edge (&edge);
 
-	if (state > state1)
+	if (state->number > state1->number)
 	  edge.type = back_edge;
 	open_edge (&edge, fgraph);
 	/* The edge source is the current node.  */
 	edge.sourcename = node_name;
-	sprintf (buff, "%d", state1);
+	sprintf (buff, "%d", state1->number);
 	edge.targetname = buff;
-	edge.color = (symbol == 0) ? red : blue;
-	edge.label = tags[symbol];
+	/* Shifts are blue, gotos are green, and error is red. */
+	if (TRANSITION_IS_ERROR (transitions, i))
+	  edge.color = red;
+	else
+	  edge.color = TRANSITION_IS_SHIFT(transitions, i) ? blue : green;
+	edge.label = symbols[symbol]->tag;
 	output_edge (&edge, fgraph);
 	close_edge (fgraph);
       }
-
-#if 0
-  if (errp)
-    {
-      int j, nerrs;
-
-      nerrs = errp->nerrs;
-
-      for (j = 0; j < nerrs; j++)
-	{
-	  if (!errp->errs[j])
-	    continue;
-	  symbol = errp->errs[j];
-	  /* If something has been added in the NODE_OBSTACK after
-	     the declaration of the label, then we need a `\n'.
-	  if (obstack_object_size (node_obstack) > node_output_size)
-	    obstack_sgrow (node_obstack, "\n");
-	    */
-	  obstack_fgrow1 (node_obstack, _("%-4s\terror (nonassociative)"),
-			  tags[symbol]);
-	}
-      if (j > 0)
-	obstack_1grow (node_obstack, '\n');
-    }
-
-  if (state_table[state].consistent && redp)
-    {
-      rule = redp->rules[0];
-      symbol = rule_table[rule].lhs;
-      /*
-      if (obstack_object_size (node_obstack) > node_output_size)
-	obstack_sgrow (node_obstack, "\n");
-	*/
-      obstack_fgrow2 (node_obstack, _("$default\treduce using rule %d (%s)"),
-		      rule, tags[symbol]);
-    }
-#endif
-
-  if (i < shiftp->nshifts)
-    for (; i < shiftp->nshifts; i++)
-      if (!SHIFT_IS_DISABLED (shiftp, i))
-	{
-	  int state1 = shiftp->shifts[i];
-	  int symbol = state_table[state1].accessing_symbol;
-
-	  new_edge (&edge);
-	  open_edge (&edge, fgraph);
-	  edge.sourcename = node_name;
-	  sprintf (buff, "%d", state1);
-	  edge.targetname = buff;
-	  edge.color = red;
-	  edge.label = tags[symbol];
-	  output_edge (&edge, fgraph);
-	  close_edge (fgraph);
-	}
 }
 
 
@@ -189,7 +162,7 @@ print_actions (int state, const char *node_name)
 `-------------------------------------------------------------*/
 
 static void
-print_state (int state)
+print_state (state_t *state)
 {
   static char name[10];
   struct obstack node_obstack;
@@ -198,9 +171,9 @@ print_state (int state)
   /* The labels of the nodes are their the items.  */
   obstack_init (&node_obstack);
   new_node (&node);
-  sprintf (name, "%d", state);
+  sprintf (name, "%d", state->number);
   node.title = name;
-  print_core (state, &node_obstack);
+  print_core (&node_obstack, state);
   obstack_1grow (&node_obstack, '\0');
   node.label = obstack_finish (&node_obstack);
 
@@ -218,10 +191,7 @@ print_state (int state)
 void
 print_graph (void)
 {
-  int i;
-
-  if (!graph_flag)
-    return;
+  state_number_t i;
 
   /* Output file.  */
   fgraph = xfopen (spec_graph_file, "w");
@@ -249,8 +219,10 @@ print_graph (void)
   output_graph (&graph, fgraph);
 
   /* Output nodes and edges. */
+  new_closure (nritems);
   for (i = 0; i < nstates; i++)
-    print_state (i);
+    print_state (states[i]);
+  free_closure ();
 
   /* Close graph. */
   close_graph (&graph, fgraph);
